@@ -97,7 +97,7 @@ class TableHandler(TableHandlerInterface):
         TableHandler.table_rename(self.table_name, temp_table)
         new_table = TableHandler(self._model, history=self.history)
         columns, old_columns = [], []
-        for column, values in self._columns.iteritems():
+        for column, values in self._columns.items():
             typname = new_columns.get(column, {}).get(
                 'typname', values['typname'])
             size = new_columns.get(column, {}).get('size', values['size'])
@@ -114,15 +114,14 @@ class TableHandler(TableHandlerInterface):
         cursor.execute('DROP TABLE "%s"' % temp_table)
         self._update_definitions()
 
-    def column_rename(self, old_name, new_name, exception=False):
-        if self.column_exist(old_name) and \
-                not self.column_exist(new_name):
-            self._recreate_table({old_name: {'name': new_name}})
-        elif exception and self.column_exist(new_name):
-            raise Exception('Unable to rename column %s.%s to %s.%s: '
-                '%s.%s already exist!'
-                % (self.table_name, old_name, self.table_name, new_name,
-                    self.table_name, new_name))
+    def column_rename(self, old_name, new_name):
+        if self.column_exist(old_name):
+            if self.column_exist(new_name):
+                self._recreate_table({old_name: {'name': new_name}})
+            else:
+                logger.warning(
+                    'Unable to rename column %s on table %s to %s.',
+                    old_name, self.table_name, new_name)
 
     def _update_definitions(self, columns=None, indexes=None):
         if columns is None and indexes is None:
@@ -244,24 +243,53 @@ class TableHandler(TableHandlerInterface):
     def drop_fk(self, column_name, table=None):
         warnings.warn('Unable to drop foreign key with SQLite backend')
 
-    def index_action(self, column_name, action='add', table=None):
-        if isinstance(column_name, basestring):
-            column_name = [column_name]
-        index_name = self.convert_name(
-            self.table_name + "_" + '_'.join(column_name) + "_index")
+    def index_action(self, columns, action='add', where='', table=None):
+        if isinstance(columns, str):
+            columns = [columns]
+
+        def stringify(column):
+            if isinstance(column, str):
+                return column
+            else:
+                return ('_'.join(
+                        map(str, (column,) + column.params))
+                    .replace('"', '')
+                    .replace('?', '__'))
+
+        name = [table or self.table_name]
+        name.append('_'.join(map(stringify, columns)))
+        if where:
+            name.append('+where')
+            name.append(stringify(where))
+        name.append('index')
+        index_name = self.convert_name('_'.join(name))
 
         cursor = Transaction().connection.cursor()
         if action == 'add':
             if index_name in self._indexes:
                 return
+            columns_quoted = []
+            for column in columns:
+                if isinstance(column, str):
+                    columns_quoted.append('"%s"' % column)
+                else:
+                    columns_quoted.append(str(column))
+            params = sum(
+                (c.params for c in columns if hasattr(c, 'params')), ())
+            if where:
+                params += where.params
+                where = ' WHERE %s' % where
+            if params:
+                warnings.warn('Unable to create index with parameters')
+                return
             cursor.execute('CREATE INDEX "' + index_name + '" '
-                'ON "' + self.table_name + '" ( ' +
-                ','.join('"' + x + '"' for x in column_name) +
-                ')')
+                'ON "' + self.table_name + '" '
+                + '(' + ','.join(columns_quoted) + ')' + where,
+                params)
             self._update_definitions(indexes=True)
         elif action == 'remove':
-            if len(column_name) == 1:
-                if self._field2module.get(column_name[0],
+            if len(columns) == 1 and isinstance(columns[0], str):
+                if self._field2module.get(columns[0],
                         self.module_name) != self.module_name:
                     return
 
@@ -282,46 +310,34 @@ class TableHandler(TableHandlerInterface):
         else:
             raise Exception('Not null action not supported!')
 
-    def add_constraint(self, ident, constraint, exception=False):
+    def add_constraint(self, ident, constraint):
         warnings.warn('Unable to add constraint with SQLite backend')
 
-    def drop_constraint(self, ident, exception=False, table=None):
+    def drop_constraint(self, ident, table=None):
         warnings.warn('Unable to drop constraint with SQLite backend')
 
-    def drop_column(self, column_name, exception=False):
+    def drop_column(self, column_name):
         if not self.column_exist(column_name):
             return
         transaction = Transaction()
         database = transaction.database
         cursor = transaction.connection.cursor()
         temp_table = '_temp_%s' % self.table_name
-        try:
-            TableHandler.table_rename(self.table_name, temp_table)
-        except Exception:
-            if exception:
-                raise
-            logger.warning('Unable to drop column: Error renaming '
-                'table %s to temp!', self.table_name)
+        TableHandler.table_rename(self.table_name, temp_table)
         new_table = TableHandler(self._model, history=self.history)
-        for name, values in self._columns.iteritems():
+        for name, values in self._columns.items():
             if name != column_name:
                 typname = values['typname']
                 size = values['size']
                 new_table._add_raw_column(
                     name, database.sql_type(typname), field_size=size)
-        columns_name = [x for x in new_table._columns.keys()]
+        columns_name = list(new_table._columns.keys())
         cursor.execute(('INSERT INTO "%s" (' +
                         ','.join('"%s"' % c for c in columns_name) +
                         ') SELECT ' +
                         ','.join('"%s"' % c for c in columns_name) + ' ' +
                         'FROM "%s"') % (self.table_name, temp_table))
-        try:
-            cursor.execute('DROP TABLE "%s"' % temp_table)
-        except Exception:
-            if exception:
-                raise
-            logger.warning('Unable to drop temporary table %s',
-                self.temp_table)
+        cursor.execute('DROP TABLE "%s"' % temp_table)
         self._update_definitions()
 
     @staticmethod
