@@ -1,17 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import base64
 import os
 from operator import itemgetter
 from collections import defaultdict
 from functools import partial
 
-from sql import Table, Null
-from sql.aggregate import Count
+from sql import Null
 
-from ..model import ModelView, ModelStorage, ModelSQL, fields
+from ..model import ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields
 from ..tools import file_open
-from .. import backend
 from ..pyson import PYSONDecoder, PYSON, Eval
 from ..transaction import Transaction
 from ..pool import Pool
@@ -27,7 +24,7 @@ __all__ = [
 EMAIL_REFKEYS = set(('cc', 'to', 'subject'))
 
 
-class Action(ModelSQL, ModelView):
+class Action(DeactivableMixin, ModelSQL, ModelView):
     "Action"
     __name__ = 'ir.action'
     name = fields.Char('Name', required=True, translate=True)
@@ -38,7 +35,6 @@ class Action(ModelSQL, ModelView):
     groups = fields.Many2Many('ir.action-res.group', 'action', 'group',
             'Groups')
     icon = fields.Many2One('ir.ui.icon', 'Icon')
-    active = fields.Boolean('Active', select=True)
 
     @classmethod
     def __setup__(cls):
@@ -51,10 +47,6 @@ class Action(ModelSQL, ModelView):
     def default_usage():
         return None
 
-    @staticmethod
-    def default_active():
-        return True
-
     @classmethod
     def write(cls, actions, values, *args):
         pool = Pool()
@@ -125,10 +117,9 @@ class ActionKeyword(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         super(ActionKeyword, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action(['keyword', 'model'], 'add')
 
     def get_groups(self, name):
@@ -230,7 +221,7 @@ class ActionKeyword(ModelSQL, ModelView):
         for action_keyword in action_keywords:
             type_ = action_keyword.action.type
             types[type_].append(action_keyword.action.id)
-        for type_, action_ids in types.iteritems():
+        for type_, action_ids in types.items():
             for value in Action.get_action_values(type_, action_ids):
                 value['keyword'] = keyword
                 keywords.append(value)
@@ -411,6 +402,8 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     direct_print = fields.Boolean('Direct Print')
     single = fields.Boolean("Single",
         help="Check if the template works only for one record.")
+    translatable = fields.Boolean("Translatable",
+        help="Uncheck to disable translations for this report.")
     template_extension = fields.Selection([
             ('odt', 'OpenDocument Text'),
             ('odp', 'OpenDocument Presentation'),
@@ -488,7 +481,7 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
             ('xpm', 'X PixMap'),
             ], translate=False,
         string='Extension', help='Leave empty for the same as template, '
-        'see unoconv documentation for compatible format')
+        'see LibreOffice documentation for compatible format')
     module = fields.Char('Module', readonly=True, select=True)
     email = fields.Char('Email',
         help='Python dictonary where keys define "to" "cc" "subject"\n'
@@ -504,65 +497,13 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         super(ActionReport, cls).__register__(module_name)
 
         transaction = Transaction()
         cursor = transaction.connection.cursor()
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         action_report = cls.__table__()
 
-        # Migration from 1.0 report_name_uniq has been removed
-        table.drop_constraint('report_name_uniq')
-
-        # Migration from 1.0 output_format (m2o) is now extension (selection)
-        if table.column_exist('output_format'):
-            outputformat = Table('ir_action_report_outputformat')
-            cursor.execute(*action_report.join(
-                    outputformat,
-                    condition=action_report.output_format == outputformat.id)
-                .select(action_report.id,
-                    where=outputformat.format == 'pdf'))
-
-            ids = [x[0] for x in cursor.fetchall()]
-            cls.write(cls.browse(ids), {'extension': 'pdf'})
-            ids = cls.search([('id', 'not in', ids)])
-            cls.write(cls.browse(ids), {'extension': 'odt'})
-
-            table.drop_column("output_format")
-            TableHandler.dropTable('ir.action.report.outputformat',
-                'ir_action_report_outputformat')
-
-        # Migrate from 2.0 remove required on extension
-        table.not_null_action('extension', action='remove')
-        cursor.execute(*action_report.update(
-                [action_report.extension],
-                [''],
-                where=action_report.extension == 'odt'))
-
-        # Migration from 2.0 report_content_data renamed into
-        # report_content_custom to remove base64 encoding
-        if (table.column_exist('report_content_data')
-                and table.column_exist('report_content_custom')):
-            limit = transaction.database.IN_MAX
-            cursor.execute(*action_report.select(
-                    Count(action_report.id)))
-            report_count, = cursor.fetchone()
-            for offset in range(0, report_count, limit):
-                cursor.execute(*action_report.select(
-                        action_report.id, action_report.report_content_data,
-                        order_by=action_report.id,
-                        limit=limit, offset=offset))
-                for report_id, report in cursor.fetchall():
-                    if report:
-                        report = fields.Binary.cast(
-                            base64.decodestring(bytes(report)))
-                        cursor.execute(*action_report.update(
-                                [action_report.report_content_custom],
-                                [report],
-                                where=action_report.id == report_id))
-            table.drop_column('report_content_data')
-
         # Migration from 3.4 remove report_name_module_uniq constraint
         table.drop_constraint('report_name_module_uniq')
 
@@ -588,6 +529,10 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def default_single(cls):
         return False
 
+    @classmethod
+    def default_translatable(cls):
+        return True
+
     @staticmethod
     def default_template_extension():
         return 'odt'
@@ -748,16 +693,10 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         cursor = Transaction().connection.cursor()
-        TableHandler = backend.get('TableHandler')
         act_window = cls.__table__()
         super(ActionActWindow, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
-
-        # Migration from 2.0: new search_value format
-        cursor.execute(*act_window.update(
-                [act_window.search_value], ['[]'],
-                where=act_window.search_value == '{}'))
+        table = cls.__table_handler__(module_name)
 
         # Migration from 3.0: auto_refresh removed
         table.drop_column('auto_refresh')
@@ -902,7 +841,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
         defaults = {
             'domain': '[]',
             'context': '{}',
-            'search_value': '{}',
+            'search_value': '[]',
             }
         for window in windows:
             pysons[window.id] = (getattr(window, field)
@@ -922,7 +861,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
         return Action.get_action_values(cls.__name__, [action_id])[0]
 
 
-class ActionActWindowView(ModelSQL, ModelView):
+class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
     "Action act window view"
     __name__ = 'ir.action.act_window.view'
     sequence = fields.Integer('Sequence', required=True)
@@ -930,26 +869,12 @@ class ActionActWindowView(ModelSQL, ModelView):
             ondelete='CASCADE')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
             ondelete='CASCADE')
-    active = fields.Boolean('Active', select=True)
 
     @classmethod
     def __setup__(cls):
         super(ActionActWindowView, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
 
-    @staticmethod
-    def default_active():
-        return True
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        super(ActionActWindowView, cls).__register__(module_name)
-        table = TableHandler(cls, module_name)
-
-        # Migration from 1.0 remove multi
-        table.drop_column('multi')
-
     @classmethod
     def create(cls, vlist):
         pool = Pool()
@@ -970,7 +895,7 @@ class ActionActWindowView(ModelSQL, ModelView):
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
 
-class ActionActWindowDomain(ModelSQL, ModelView):
+class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
     "Action act window domain"
     __name__ = 'ir.action.act_window.domain'
     name = fields.Char('Name', translate=True)
@@ -979,7 +904,6 @@ class ActionActWindowDomain(ModelSQL, ModelView):
     count = fields.Boolean('Count')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
         select=True, required=True, ondelete='CASCADE')
-    active = fields.Boolean('Active')
 
     @classmethod
     def __setup__(cls):
@@ -990,10 +914,6 @@ class ActionActWindowDomain(ModelSQL, ModelView):
                     '"%(domain)s" on action "%(action)s".'),
                 })
 
-    @staticmethod
-    def default_active():
-        return True
-
     @classmethod
     def default_count(cls):
         return False
