@@ -1,14 +1,13 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import polib
+import os
 import xml.dom.minidom
 from difflib import SequenceMatcher
-import os
 from hashlib import md5
-from lxml import etree
-from itertools import izip
 from io import BytesIO
+from lxml import etree
 
+import polib
 from sql import Column, Null, Literal
 from sql.functions import Substring, Position
 from sql.conditionals import Case
@@ -22,15 +21,15 @@ from relatorio.templates.opendocument import get_zip_file
 from ..model import ModelView, ModelSQL, fields, Unique
 from ..wizard import Wizard, StateView, StateTransition, StateAction, \
     Button
-from ..tools import file_open, reduce_ids, grouped_slice, cursor_dict, \
-        get_parent_language as get_parent
-from .. import backend
+from ..tools import file_open, reduce_ids, grouped_slice, cursor_dict
 from ..pyson import PYSONEncoder, Eval
 from ..transaction import Transaction
 from ..pool import Pool
 from ..cache import Cache
 from ..config import config
 
+from trytond.ir.lang import get_parent_language as get_parent
+
 __all__ = ['Translation',
     'TranslationSetStart', 'TranslationSetSucceed', 'TranslationSet',
     'TranslationCleanStart', 'TranslationCleanSucceed', 'TranslationClean',
@@ -97,48 +96,20 @@ class Translation(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         transaction = Transaction()
         cursor = transaction.connection.cursor()
         ir_translation = cls.__table__()
-        table = TableHandler(cls, module_name)
-        # Migration from 1.8: new field src_md5
-        src_md5_exist = table.column_exist('src_md5')
-        if not src_md5_exist:
-            table.add_column('src_md5', cls.src_md5._sql_type)
-        table.drop_constraint('translation_uniq')
-        table.index_action(['lang', 'type', 'name', 'src'], 'remove')
+        table = cls.__table_handler__(module_name)
 
         super(Translation, cls).__register__(module_name)
 
-        # Migration from 1.8: fill new field src_md5
-        if not src_md5_exist:
-            offset = 0
-            limit = transaction.database.IN_MAX
-            translations = True
-            while translations:
-                translations = cls.search([], offset=offset, limit=limit)
-                offset += limit
-                for translation in translations:
-                    src_md5 = cls.get_src_md5(translation.src)
-                    cls.write([translation], {
-                        'src_md5': src_md5,
-                    })
-            table = TableHandler(cls, module_name)
-            table.not_null_action('src_md5', action='add')
-
-        # Migration from 2.2 and 2.8
-        cursor.execute(*ir_translation.update([ir_translation.res_id],
-                [-1], where=(ir_translation.res_id == Null)
-                | (ir_translation.res_id == 0)))
-
         # Migration from 3.8: rename odt type in report
         cursor.execute(*ir_translation.update(
                 [ir_translation.type],
                 ['report'],
                 where=ir_translation.type == 'odt'))
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action(['lang', 'type', 'name'], 'add')
 
     @classmethod
@@ -258,7 +229,7 @@ class Translation(ModelSQL, ModelView):
                             [[trans_name, 'en', 'selection', val, val_md5,
                                     '', module_name, False, -1]]))
 
-        for field_name, field in model._fields.iteritems():
+        for field_name, field in model._fields.items():
             trans_name = model.__name__ + ',' + field_name
             update_insert_field(field, trans_name)
             update_insert_help(field, trans_name)
@@ -325,7 +296,7 @@ class Translation(ModelSQL, ModelView):
                         where=ir_translation.id ==
                         trans_buttons[trans_name]['id']))
 
-        for state_name, state in wizard.states.iteritems():
+        for state_name, state in wizard.states.items():
             if not isinstance(state, StateView):
                 continue
             for button in state.buttons:
@@ -394,20 +365,20 @@ class Translation(ModelSQL, ModelView):
         Model = pool.get('ir.model')
 
         translations, to_fetch = {}, []
-        name = unicode(name)
-        ttype = unicode(ttype)
-        lang = unicode(lang)
+        name = str(name)
+        ttype = str(ttype)
+        lang = str(lang)
         if name.split(',')[0] in ('ir.model.field', 'ir.model'):
             field_name = name.split(',')[1]
             with Transaction().set_context(_check_access=False):
                 if name.split(',')[0] == 'ir.model.field':
                     if field_name == 'field_description':
-                        ttype = u'field'
+                        ttype = 'field'
                     else:
-                        ttype = u'help'
+                        ttype = 'help'
                     records = ModelFields.browse(ids)
                 else:
-                    ttype = u'model'
+                    ttype = 'model'
                     records = Model.browse(ids)
 
             trans_args = []
@@ -523,7 +494,7 @@ class Translation(ModelSQL, ModelView):
                     translations[translation.name] = translation
 
                 to_save = []
-                for record, value in izip(records, values):
+                for record, value in zip(records, values):
                     translation = translations.get(get_name(record))
                     if not translation:
                         translation = cls()
@@ -564,7 +535,7 @@ class Translation(ModelSQL, ModelView):
                         ).append(translation)
 
             to_save = []
-            for record, value in izip(records, values):
+            for record, value in zip(records, values):
                 translation = translations.get(record.id)
                 if not translation:
                     translation = cls()
@@ -631,11 +602,11 @@ class Translation(ModelSQL, ModelView):
 
         to_cache = []
         for name, ttype, lang, source in args:
-            name = unicode(name)
-            ttype = unicode(ttype)
-            lang = unicode(lang)
+            name = str(name)
+            ttype = str(ttype)
+            lang = str(lang)
             if source is not None:
-                source = unicode(source)
+                source = str(source)
             trans = cls._translation_cache.get((name, ttype, lang, source), -1)
             if trans != -1:
                 res[(name, ttype, lang, source)] = trans
@@ -748,7 +719,7 @@ class Translation(ModelSQL, ModelView):
     def translation_import(cls, lang, module, po_path):
         pool = Pool()
         ModelData = pool.get('ir.model.data')
-        if isinstance(po_path, basestring):
+        if isinstance(po_path, str):
             po_path = [po_path]
         models_data = ModelData.search([
                 ('module', '=', module),
@@ -872,7 +843,7 @@ class Translation(ModelSQL, ModelView):
                                 to_save.append(old_translation)
                             else:
                                 translations.add(old_translation)
-        cls.save(filter(None, to_save))
+        cls.save([_f for _f in to_save if _f])
         translations |= set(to_save)
 
         if translations:
@@ -938,7 +909,7 @@ class Translation(ModelSQL, ModelView):
 
         if pofile:
             pofile.sort()
-            return unicode(pofile).encode('utf-8')
+            return str(pofile).encode('utf-8')
         else:
             return
 
@@ -1037,7 +1008,7 @@ class TranslationSet(Wizard):
             reports = Report.browse(context.get('active_ids', []))
         elif context.get('active_model', 'ir.ui.menu') == 'ir.ui.menu':
             with Transaction().set_context(active_test=False):
-                reports = Report.search([])
+                reports = Report.search([('translatable', '=', True)])
         else:
             return
 
@@ -1282,6 +1253,7 @@ class TranslationClean(Wizard):
         with Transaction().set_context(active_test=False):
             if not Report.search([
                         ('report_name', '=', translation.name),
+                        ('translatable', '=', True),
                         ]):
                 return True
 
@@ -1391,15 +1363,15 @@ class TranslationClean(Wizard):
             except KeyError:
                 pass
         if Model:
-            errors = Model._error_messages.values()
+            errors = list(Model._error_messages.values())
             if issubclass(Model, ModelSQL):
-                errors += Model._sql_error_messages.values()
+                errors += list(Model._sql_error_messages.values())
                 for _, _, error in Model._sql_constraints:
                     errors.append(error)
             if translation.src not in errors:
                 return True
         elif Wizard:
-            errors = Wizard._error_messages.values()
+            errors = list(Wizard._error_messages.values())
             if translation.src not in errors:
                 return True
         else:
