@@ -10,6 +10,11 @@ class Integer(Field):
     _type = 'integer'
     _sql_type = 'INTEGER'
 
+    def sql_format(self, value):
+        if value is None:
+            return None
+        return int(str(value))
+
 
 class BigInteger(Integer):
     '''
