@@ -7,7 +7,7 @@ import csv
 import warnings
 
 from decimal import Decimal
-from itertools import islice, ifilter, chain, izip
+from itertools import islice, chain
 from functools import reduce, wraps
 from operator import itemgetter
 from collections import defaultdict
@@ -22,7 +22,6 @@ from trytond.config import config
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.cache import LRUDict, LRUDictTransaction, freeze
-from trytond import backend
 from trytond.rpc import RPC
 from .modelview import ModelView
 from .descriptors import dualmethod
@@ -43,6 +42,13 @@ def cache_size():
         config.getint('cache', 'record'))
 
 
+def is_leaf(expression):
+    return (isinstance(expression, (list, tuple))
+        and len(expression) > 2
+        and isinstance(expression[1], str)
+        and expression[1] in OPERATORS)  # TODO remove OPERATORS test
+
+
 class ModelStorage(Model):
     """
     Define a model with storage capability in Tryton.
@@ -61,14 +67,14 @@ class ModelStorage(Model):
         if issubclass(cls, ModelView):
             cls.__rpc__.update({
                     'create': RPC(readonly=False,
-                        result=lambda r: map(int, r)),
+                        result=lambda r: list(map(int, r))),
                     'read': RPC(),
                     'write': RPC(readonly=False,
                         instantiate=slice(0, None, 2)),
                     'delete': RPC(readonly=False, instantiate=0),
                     'copy': RPC(readonly=False, instantiate=0, unique=False,
-                        result=lambda r: map(int, r)),
-                    'search': RPC(result=lambda r: map(int, r)),
+                        result=lambda r: list(map(int, r))),
+                    'search': RPC(result=lambda r: list(map(int, r))),
                     'search_count': RPC(),
                     'search_read': RPC(),
                     'export_data': RPC(instantiate=0, unique=False),
@@ -97,7 +103,7 @@ class ModelStorage(Model):
 
         ModelAccess.check(cls.__name__, 'create')
 
-        all_fields = list(set(chain(*(v.iterkeys() for v in vlist))))
+        all_fields = list(set(chain(*(iter(v.keys()) for v in vlist))))
         ModelFieldAccess.check(cls.__name__, all_fields, 'write')
 
         # Increase transaction counter
@@ -161,7 +167,7 @@ class ModelStorage(Model):
                 cls.raise_user_error('write_xml_record',
                         error_description='xml_record_desc')
             all_records += records
-            all_fields.update(values.iterkeys())
+            all_fields.update(values.keys())
 
         ModelAccess.check(cls.__name__, 'write')
         ModelFieldAccess.check(cls.__name__, all_fields, 'write')
@@ -176,13 +182,14 @@ class ModelStorage(Model):
                 local_cache.clear()
 
         # Clean transaction cache
-        for cache in Transaction().cache.itervalues():
+        for cache in Transaction().cache.values():
             if cls.__name__ in cache:
                 for record in all_records:
                     if record.id in cache[cls.__name__]:
                         cache[cls.__name__][record.id].clear()
 
     @classmethod
+    @without_check_access
     def trigger_write_get_eligibles(cls, records):
         '''
         Return eligible records for write actions by triggers
@@ -207,7 +214,7 @@ class ModelStorage(Model):
         eligibles is a dictionary of the lists of eligible records by triggers
         '''
         Trigger = Pool().get('ir.trigger')
-        for trigger, records in eligibles.iteritems():
+        for trigger, records in eligibles.items():
             triggered = []
             for record in records:
                 if Trigger.eval(trigger, record):
@@ -215,6 +222,11 @@ class ModelStorage(Model):
             if triggered:
                 Trigger.trigger_action(triggered, trigger)
 
+    @classmethod
+    def index_set_field(cls, name):
+        "Returns the index sort order of the field set calls."
+        return 0
+
     @classmethod
     def delete(cls, records):
         '''
@@ -231,8 +243,8 @@ class ModelStorage(Model):
         Transaction().counter += 1
 
         # Clean transaction cache
-        for cache in Transaction().cache.values():
-            for cache in (cache, cache.get('_language_cache', {}).values()):
+        for cache in list(Transaction().cache.values()):
+            for cache in (cache, list(cache.get('_language_cache', {}).values())):
                 if cls.__name__ in cache:
                     for record in records:
                         if record.id in cache[cls.__name__]:
@@ -265,18 +277,21 @@ class ModelStorage(Model):
         Lang = pool.get('ir.lang')
         if default is None:
             default = {}
-
-        if 'state' not in default:
-            if 'state' in cls._defaults:
-                default['state'] = cls._defaults['state']()
+        else:
+            default = default.copy()
 
         def is_readonly(Model):
             return (not issubclass(Model, ModelStorage)
-                or (hasattr(Model, 'table_query')
-                    and Model.table_query()))
+                or callable(getattr(Model, 'table_query', None)))
 
-        def convert_data(field_defs, data):
-            data = data.copy()
+        def get_default(name):
+            prefix = name + '.'
+            return {name[len(prefix):]: value
+                for name, value in default.items()
+                if name.startswith(prefix)}
+
+        def convert_data(field_defs, origin):
+            data = origin.copy()
             for field_name in field_defs:
                 ftype = field_defs[field_name]['type']
                 field = cls._fields[field_name]
@@ -290,7 +305,10 @@ class ModelStorage(Model):
                     del data[field_name]
 
                 if field_name in default:
-                    data[field_name] = default[field_name]
+                    if callable(default[field_name]):
+                        data[field_name] = default[field_name](origin)
+                    else:
+                        data[field_name] = default[field_name]
                 if (isinstance(field, fields.Function)
                         and not isinstance(field, fields.MultiValue)):
                     del data[field_name]
@@ -304,7 +322,9 @@ class ModelStorage(Model):
                     if is_readonly(field.get_target()):
                         del data[field_name]
                     elif data[field_name]:
-                        data[field_name] = [('copy', data[field_name])]
+                        data[field_name] = [(
+                                'copy', data[field_name],
+                                get_default(field_name))]
                 elif ftype == 'many2many':
                     if is_readonly(pool.get(field.relation_name)):
                         del data[field_name]
@@ -316,17 +336,17 @@ class ModelStorage(Model):
 
         # Reset MPTT field to the default value
         mptt = set()
-        for field in cls._fields.itervalues():
+        for field in cls._fields.values():
             if (isinstance(field, fields.Many2One)
                     and field.model_name == cls.__name__
                     and field.left and field.right):
                 mptt.add(field.left)
                 mptt.add(field.right)
-        fields_names = [n for n, f in cls._fields.iteritems()
+        fields_names = [n for n, f in cls._fields.items()
             if (not isinstance(f, fields.Function)
                 or isinstance(f, fields.MultiValue))
             and n not in mptt]
-        ids = map(int, records)
+        ids = list(map(int, records))
         datas = cls.read(ids, fields_names=fields_names)
         datas = dict((d['id'], d) for d in datas)
         field_defs = cls.fields_get(fields_names=fields_names)
@@ -335,10 +355,10 @@ class ModelStorage(Model):
             data = convert_data(field_defs, datas[id])
             to_create.append(data)
         new_records = cls.create(to_create)
-        id2new_record = dict(izip(ids, new_records))
+        id2new_record = dict(zip(ids, new_records))
 
         fields_translate = {}
-        for field_name, field in field_defs.iteritems():
+        for field_name, field in field_defs.items():
             if field_name in cls._fields and \
                     getattr(cls._fields[field_name], 'translate', False):
                 fields_translate[field_name] = field
@@ -348,13 +368,13 @@ class ModelStorage(Model):
                 ('translatable', '=', True),
                 ])
             if langs:
+                fields_names = list(fields_translate.keys()) + ['id']
                 for lang in langs:
                     # Prevent fuzzing translations when copying as the terms
                     # should be the same.
                     with Transaction().set_context(language=lang.code,
                             fuzzy_translation=False):
-                        datas = cls.read(ids,
-                                fields_names=fields_translate.keys() + ['id'])
+                        datas = cls.read(ids, fields_names=fields_names)
                         to_write = []
                         for data in datas:
                             to_write.append([id2new_record[data['id']]])
@@ -368,6 +388,36 @@ class ModelStorage(Model):
         '''
         Return a list of records that match the domain.
         '''
+        pool = Pool()
+        transaction = Transaction()
+        ModelAccess = pool.get('ir.model.access')
+        ModelFieldAccess = pool.get('ir.model.field.access')
+
+        ModelAccess.check(cls.__name__, 'read')
+
+        def check(domain, cls, to_check):
+            if is_leaf(domain):
+                local, relate = (domain[0].split('.', 1) + [None])[:2]
+                to_check[cls.__name__].add(local)
+                if relate:
+                    if len(domain) >= 4:
+                        target = pool.get(domain[3])
+                    else:
+                        target = cls._fields[local].get_target()
+                    target_domain = [(relate,) + tuple(domain[1:])]
+                    check(target_domain, target, to_check)
+            elif not domain:
+                return
+            else:
+                i = 1 if domain[0] in ['OR', 'AND'] else 0
+                for d in domain[i:]:
+                    check(d, cls, to_check)
+        if transaction.user and transaction.context.get('_check_access'):
+            to_check = defaultdict(set)
+            check(domain, cls, to_check)
+            for name, fields_names in to_check.items():
+                ModelAccess.check(name, 'read')
+                ModelFieldAccess.check(name, fields_names, 'read')
         if count:
             return 0
         return []
@@ -392,10 +442,10 @@ class ModelStorage(Model):
         records = cls.search(domain, offset=offset, limit=limit, order=order)
 
         if not fields_names:
-            fields_names = cls._fields.keys()
+            fields_names = list(cls._fields.keys())
         if 'id' not in fields_names:
             fields_names.append('id')
-        rows = cls.read(map(int, records), fields_names)
+        rows = cls.read(list(map(int, records)), fields_names)
         index = {r.id: i for i, r in enumerate(records)}
         rows.sort(key=lambda r: index[r['id']])
         return rows
@@ -417,19 +467,16 @@ class ModelStorage(Model):
             while i < len(domain):
                 arg = domain[i]
                 # add test for xmlrpc that doesn't handle tuple
-                if (isinstance(arg, tuple)
-                        or (isinstance(arg, list)
-                            and len(arg) > 2
-                            and arg[1] in OPERATORS)):
+                if is_leaf(arg):
                     if arg[0] == 'active':
                         active_found = True
                 elif isinstance(arg, list):
                     domain[i] = process(domain[i])
                 i += 1
             if not active_found:
-                if (domain and ((isinstance(domain[0], basestring)
+                if (domain and ((isinstance(domain[0], str)
                                 and domain[0] == 'AND')
-                            or (not isinstance(domain[0], basestring)))):
+                            or (not isinstance(domain[0], str)))):
                     domain.append(('active', '=', True))
                 else:
                     domain = ['AND', domain, ('active', '=', True)]
@@ -444,7 +491,7 @@ class ModelStorage(Model):
         rec_name = self._rec_name
         if rec_name not in self._fields:
             rec_name = 'id'
-        return unicode(getattr(self, rec_name))
+        return str(getattr(self, rec_name))
 
     @classmethod
     def search_rec_name(cls, name, clause):
@@ -473,7 +520,7 @@ class ModelStorage(Model):
         Return a list of instance for the ids
         '''
         transaction = Transaction()
-        ids = map(int, ids)
+        ids = list(map(int, ids))
         local_cache = LRUDictTransaction(cache_size())
         transaction_cache = transaction.get_cache()
         return [cls(x, _ids=ids,
@@ -530,7 +577,7 @@ class ModelStorage(Model):
                         child_lines = ModelStorage.__export_row(child_record,
                                 child_fields_names)
                         if first:
-                            for child_fpos in xrange(len(fields_names)):
+                            for child_fpos in range(len(fields_names)):
                                 if child_lines and child_lines[0][child_fpos]:
                                     data[child_fpos] = \
                                         child_lines[0][child_fpos]
@@ -592,8 +639,8 @@ class ModelStorage(Model):
                 return None
             res = []
             Relation = pool.get(relation)
-            for word in csv.reader(value.splitlines(), delimiter=',',
-                    quoting=csv.QUOTE_NONE, escapechar='\\').next():
+            for word in next(csv.reader(value.splitlines(), delimiter=',',
+                    quoting=csv.QUOTE_NONE, escapechar='\\')):
                 res2 = Relation.search([
                     ('rec_name', '=', word),
                     ], limit=2)
@@ -641,8 +688,8 @@ class ModelStorage(Model):
             relation = None
             ftype = fields_def[field[-1][:-3]]['type']
             if ftype == 'many2many':
-                value = csv.reader(value.splitlines(), delimiter=',',
-                        quoting=csv.QUOTE_NONE, escapechar='\\').next()
+                value = next(csv.reader(value.splitlines(), delimiter=',',
+                        quoting=csv.QUOTE_NONE, escapechar='\\'))
             elif ftype == 'reference':
                 try:
                     relation, value = value.split(',', 1)
@@ -736,7 +783,7 @@ class ModelStorage(Model):
                 (newrow, max2, _) = res
                 nbrmax = max(nbrmax, max2)
                 reduce(lambda x, y: x and y, newrow)
-                row[field] = (reduce(lambda x, y: x or y, newrow.values()) and
+                row[field] = (reduce(lambda x, y: x or y, list(newrow.values())) and
                          [('create', [newrow])]) or []
                 i = max2
                 while (position + i) < len(data):
@@ -751,12 +798,12 @@ class ModelStorage(Model):
                     (newrow, max2, _) = \
                         process_lines(data, prefix + [field], newfd,
                             position + i)
-                    if reduce(lambda x, y: x or y, newrow.values()):
+                    if reduce(lambda x, y: x or y, list(newrow.values())):
                         row[field].append(('create', [newrow]))
                     i += max2
                     nbrmax = max(nbrmax, i)
             if prefix_len == 0:
-                for i in xrange(max(nbrmax, 1)):
+                for i in range(max(nbrmax, 1)):
                     data.pop(0)
             return (row, nbrmax, translate)
 
@@ -778,8 +825,8 @@ class ModelStorage(Model):
         for language in languages:
             translated = [t.get(language, {}) for t in translations]
             with Transaction().set_context(language=language):
-                cls.write(*chain(*ifilter(itemgetter(1),
-                            izip(([r] for r in new_records), translated))))
+                cls.write(*chain(*filter(itemgetter(1),
+                            zip(([r] for r in new_records), translated))))
         return len(new_records)
 
     @classmethod
@@ -796,12 +843,13 @@ class ModelStorage(Model):
         """
         ModelData = Pool().get('ir.model.data')
         # Allow root user to update/delete
-        if Transaction().user == 0:
+        if (Transaction().user == 0
+                or not ModelData.has_model(cls.__name__)):
             return True
         with Transaction().set_context(_check_access=False):
             models_data = ModelData.search([
                 ('model', '=', cls.__name__),
-                ('db_id', 'in', map(int, records)),
+                ('db_id', 'in', list(map(int, records))),
                 ])
             if not models_data:
                 return True
@@ -811,54 +859,11 @@ class ModelStorage(Model):
                 if not model_data.values or model_data.noupdate:
                     continue
                 xml_values = ModelData.load_values(model_data.values)
-                for key, val in values.iteritems():
+                for key, val in values.items():
                     if key in xml_values and val != xml_values[key]:
                         return False
         return True
 
-    @classmethod
-    def check_recursion(cls, records, parent='parent', rec_name='rec_name'):
-        '''
-        Function that checks if there is no recursion in the tree
-        composed with parent as parent field name.
-        '''
-        parent_type = cls._fields[parent]._type
-
-        if parent_type not in ('many2one', 'many2many', 'one2one'):
-            raise Exception(
-                'Unsupported field type "%s" for field "%s" on "%s"'
-                % (parent_type, parent, cls.__name__))
-
-        visited = set()
-
-        for record in records:
-            walked = set()
-            walker = getattr(record, parent)
-            while walker:
-                if parent_type == 'many2many':
-                    for walk in walker:
-                        walked.add(walk.id)
-                        if walk.id == record.id:
-                            parent_rec_name = ', '.join(getattr(r, rec_name)
-                                for r in getattr(record, parent))
-                            cls.raise_user_error('recursion_error', {
-                                    'rec_name': getattr(record, rec_name),
-                                    'parent_rec_name': parent_rec_name,
-                                    })
-                    walker = list(chain(*(getattr(walk, parent)
-                                for walk in walker if walk.id not in visited)))
-                else:
-                    walked.add(walker.id)
-                    if walker.id == record.id:
-                        cls.raise_user_error('recursion_error', {
-                                'rec_name': getattr(record, rec_name),
-                                'parent_rec_name': getattr(getattr(record,
-                                        parent), rec_name)
-                                })
-                    walker = (getattr(walker, parent) not in visited
-                        and getattr(walker, parent))
-            visited.update(walked)
-
     @classmethod
     def _get_error_args(cls, field_name):
         pool = Pool()
@@ -885,6 +890,7 @@ class ModelStorage(Model):
         pass
 
     @classmethod
+    @without_check_access
     def _validate(cls, records, field_names=None):
         pool = Pool()
         # Ensure that records are readable
@@ -924,7 +930,7 @@ class ModelStorage(Model):
                         if is_pyson(i):
                             return True
             if isinstance(test, dict):
-                for key, value in test.items():
+                for key, value in list(test.items()):
                     if isinstance(value, PYSON):
                         return True
                     if isinstance(value, (list, tuple, dict)):
@@ -955,13 +961,34 @@ class ModelStorage(Model):
                     env['active_id'] = record.id
                     domain = freeze(PYSONDecoder(env).decode(pyson_domain))
                     domains[domain].append(record)
+                # Select strategy depending if it is closer to one domain per
+                # record or one domain for all records
+                if len(domains) > len(records) * 0.5:
+                    # Do not use IN_MAX to let spaces for the pyson domain
+                    in_max = Transaction().database.IN_MAX
+                    count = in_max // 10
+                    new_domains = {}
+                    for sub_domains in grouped_slice(list(domains.keys()), count):
+                        grouped_domain = ['OR']
+                        grouped_records = []
+                        for d in sub_domains:
+                            sub_records = domains[d]
+                            grouped_records.extend(sub_records)
+                            relations = relation_domain(field, sub_records)
+                            if len(relations) > in_max:
+                                break
+                            grouped_domain.append(
+                                [('id', 'in', [r.id for r in relations]), d])
+                        new_domains[freeze(grouped_domain)] = grouped_records
+                    else:
+                        domains = new_domains
             else:
                 domains[freeze(field.domain)].extend(records)
 
-            for domain, sub_records in domains.iteritems():
+            for domain, sub_records in domains.items():
                 validate_relation_domain(field, sub_records, Relation, domain)
 
-        def validate_relation_domain(field, records, Relation, domain):
+        def relation_domain(field, records):
             if field._type in ('many2one', 'one2many', 'many2many', 'one2one'):
                 relations = set()
                 for record in records:
@@ -973,6 +1000,10 @@ class ModelStorage(Model):
             else:
                 # Cache alignment is not a problem
                 relations = set(records)
+            return relations
+
+        def validate_relation_domain(field, records, Relation, domain):
+            relations = relation_domain(field, records)
             if relations:
                 for sub_relations in grouped_slice(relations):
                     sub_relations = set(sub_relations)
@@ -987,26 +1018,25 @@ class ModelStorage(Model):
                             error_args=cls._get_error_args(field.name))
 
         field_names = set(field_names or [])
-        function_fields = {name for name, field in cls._fields.iteritems()
+        function_fields = {name for name, field in cls._fields.items()
             if isinstance(field, fields.Function)}
         ctx_pref['active_test'] = False
         with Transaction().set_context(ctx_pref):
-            for field_name, field in cls._fields.iteritems():
+            for field_name, field in cls._fields.items():
                 depends = set(field.depends)
                 if (field_names
                         and field_name not in field_names
                         and not (depends & field_names)
                         and not (depends & function_fields)):
                     continue
-                if isinstance(field, fields.Function) and \
-                        not field.setter:
+                if isinstance(field, fields.Function):
                     continue
 
                 validate_domain(field)
 
                 def required_test(value, field_name, field):
                     if (isinstance(value, (type(None), type(False), list,
-                                    tuple, basestring, dict))
+                                    tuple, str, dict))
                             and not value):
                         cls.raise_user_error('required_validation_record',
                             error_args=cls._get_error_args(field_name))
@@ -1076,7 +1106,7 @@ class ModelStorage(Model):
                         exp = Decimal('.'.join(['0', '0' * digits[1]]))
                         if value.quantize(exp) != value:
                             raise_user_error(value)
-                    elif backend.name() != 'mysql':
+                    else:
                         if not (round(value, digits[1]) == float(value)):
                             raise_user_error(value)
                 # validate digits
@@ -1261,8 +1291,8 @@ class ModelStorage(Model):
             FieldAccess = Pool().get('ir.model.field.access')
             fread_accesses = {}
             fread_accesses.update(FieldAccess.check(self.__name__,
-                self._fields.keys(), 'read', access=True))
-            to_remove = set(x for x, y in fread_accesses.iteritems()
+                list(self._fields.keys()), 'read', access=True))
+            to_remove = set(x for x, y in fread_accesses.items()
                     if not y and x != name)
 
             threshold = config.getint('cache', 'field')
@@ -1281,20 +1311,20 @@ class ModelStorage(Model):
                 fname, field = item
                 return fname in self._fields
 
-            ifields = ifilter(to_load,
-                ifilter(not_cached,
-                    self._fields.iteritems()))
+            ifields = filter(to_load,
+                filter(not_cached,
+                    iter(self._fields.items())))
             ifields = islice(ifields, 0, threshold)
             ffields.update(ifields)
 
         # add datetime_field
-        for field in ffields.values():
+        for field in list(ffields.values()):
             if hasattr(field, 'datetime_field') and field.datetime_field:
                 datetime_field = self._fields[field.datetime_field]
                 ffields[field.datetime_field] = datetime_field
 
         # add depends of field with context
-        for field in ffields.values():
+        for field in list(ffields.values()):
             if field.context:
                 eval_fields = fields.get_eval_fields(field.context)
                 for context_field_name in eval_fields:
@@ -1317,7 +1347,7 @@ class ModelStorage(Model):
         index = self._ids.index(self.id)
         ids = chain(islice(self._ids, index, None),
             islice(self._ids, 0, max(index - 1, 0)))
-        ids = islice(unique(ifilter(filter_, ids)),
+        ids = islice(unique(filter(filter_, ids)),
             self._transaction.database.IN_MAX)
 
         def instantiate(field, value, data):
@@ -1384,10 +1414,10 @@ class ModelStorage(Model):
                     for i in ids
                     if i in self._cache and name in self._cache[i]]
             else:
-                read_data = self.read(list(ids), ffields.keys())
+                read_data = self.read(list(ids), list(ffields.keys()))
             # create browse records for 'remote' models
             for data in read_data:
-                for fname, field in ffields.iteritems():
+                for fname, field in ffields.items():
                     fvalue = data[fname]
                     if field._type in ('many2one', 'one2one', 'one2many',
                             'many2many', 'reference'):
@@ -1416,7 +1446,7 @@ class ModelStorage(Model):
         values = {}
         if not self._values:
             return values
-        for fname, value in self._values.iteritems():
+        for fname, value in self._values.items():
             field = self._fields[fname]
             if field._type in ('many2one', 'one2one', 'reference'):
                 if value:
@@ -1500,7 +1530,7 @@ class ModelStorage(Model):
                         transaction.set_context(context):
                     if to_create:
                         news = cls.create([save_values[r] for r in to_create])
-                        for record, new in izip(to_create, news):
+                        for record, new in zip(to_create, news):
                             record._ids.remove(record.id)
                             record._id = new.id
                             record._ids.append(record.id)
@@ -1544,7 +1574,7 @@ class EvalEnvironment(dict):
     def __getattr__(self, item):
         try:
             return self.__getitem__(item)
-        except KeyError, exception:
+        except KeyError as exception:
             raise AttributeError(*exception.args)
 
     def get(self, item, default=None):
@@ -1554,5 +1584,5 @@ class EvalEnvironment(dict):
             pass
         return super(EvalEnvironment, self).get(item, default)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self._record)
