@@ -106,7 +106,7 @@ class ModelView(Model):
         from trytond.modules import create_graph, get_module_list
         if ModelView.__modules_list:
             return ModelView.__modules_list
-        graph = create_graph(get_module_list())[0]
+        graph = create_graph(get_module_list())
         ModelView.__modules_list = [x.name for x in graph] + [None]
         return ModelView.__modules_list
 
@@ -119,64 +119,85 @@ class ModelView(Model):
         cls.__rpc__['on_change_with'] = RPC(instantiate=0)
         cls._buttons = {}
 
-        if hasattr(cls, '__depend_methods'):
-            cls.__depend_methods = cls.__depend_methods.copy()
-        else:
-            cls.__depend_methods = collections.defaultdict(set)
-
-        if hasattr(cls, '__change_buttons'):
-            cls.__change_buttons = cls.__change_buttons.copy()
-        else:
-            cls.__change_buttons = collections.defaultdict(set)
-
-        def setup_field(field, field_name):
-            for attribute in ('on_change', 'on_change_with', 'autocomplete',
-                    'selection_change_with'):
-                if attribute == 'selection_change_with':
-                    if isinstance(
-                            getattr(field, 'selection', None), basestring):
-                        function_name = field.selection
-                    else:
-                        continue
-                else:
-                    function_name = '%s_%s' % (attribute, field_name)
-                if not getattr(cls, function_name, None):
-                    continue
-                # Search depends on all parent class because field has been
-                # copied with the original definition
-                for parent_cls in cls.__mro__:
-                    function = getattr(parent_cls, function_name, None)
-                    if not function:
-                        continue
-                    if getattr(function, 'depends', None):
-                        setattr(field, attribute,
-                            getattr(field, attribute) | function.depends)
-                    if getattr(function, 'depend_methods', None):
-                        cls.__depend_methods[(field_name, attribute)] |= \
-                            function.depend_methods
-                function = getattr(cls, function_name, None)
-                if (attribute == 'on_change'
-                        and not getattr(function, 'on_change', None)):
-                    # Decorate on_change to always return self
-                    setattr(cls, function_name, on_change(function))
-
-        def setup_callable(function, name):
-            if hasattr(function, 'change'):
-                cls.__change_buttons[name] |= function.change
-
+        fields_ = {}
+        callables = {}
         for name in dir(cls):
+            if name.startswith('__'):
+                continue
             attr = getattr(cls, name)
             if isinstance(attr, fields.Field):
-                setup_field(attr, name)
-            elif isinstance(attr, collections.Callable):
-                setup_callable(attr, name)
+                fields_[name] = attr
+            elif callable(attr):
+                callables[name] = attr
+
+        methods = {
+            'depends': collections.defaultdict(set),
+            'depend_methods': collections.defaultdict(set),
+            'change': collections.defaultdict(set),
+            }
+        cls.__change_buttons = methods['change']
+
+        def get_callable_attributes(name, method):
+            for parent_cls in cls.__mro__:
+                parent_meth = getattr(parent_cls, name, None)
+                if not parent_meth:
+                    continue
+                for attr in ['depends', 'depend_methods', 'change']:
+                    parent_value = getattr(parent_meth, attr, None)
+                    if parent_value:
+                        methods[attr][name] |= parent_value
+
+        for name, method in callables.items():
+            get_callable_attributes(name, method)
+
+        def setup_field(field_name, field, attribute):
+            if attribute == 'selection_change_with':
+                if isinstance(
+                        getattr(field, 'selection', None), str):
+                    function_name = field.selection
+                else:
+                    return
+            else:
+                function_name = '%s_%s' % (attribute, field_name)
+            if not getattr(cls, function_name, None):
+                return
+
+            function = getattr(cls, function_name, None)
+            setattr(field, attribute,
+                getattr(field, attribute) | methods['depends'][function_name])
+
+            meth_names = list(methods['depend_methods'][function_name])
+            meth_done = set()
+            while meth_names:
+                meth_name = meth_names.pop()
+                assert callable(getattr(cls, meth_name)), \
+                    "%s.%s not callable" % (cls, meth_name)
+                setattr(field, attribute,
+                    getattr(field, attribute) | methods['depends'][meth_name])
+                meth_names += list(
+                    methods['depend_methods'][meth_name] - meth_done)
+                meth_done.add(meth_name)
+
+            if (attribute == 'on_change'
+                    and not getattr(function, 'on_change', None)):
+                # Decorate on_change to always return self
+                setattr(cls, function_name, on_change(function))
+
+        for name, field in fields_.items():
+            for attribute in [
+                    'on_change',
+                    'on_change_with',
+                    'autocomplete',
+                    'selection_change_with',
+                    ]:
+                setup_field(name, field, attribute)
 
     @classmethod
     def __post_setup__(cls):
         super(ModelView, cls).__post_setup__()
 
         # Update __rpc__
-        for field_name, field in cls._fields.iteritems():
+        for field_name, field in cls._fields.items():
             field.set_rpc(cls)
 
         for button in cls._buttons:
@@ -187,16 +208,6 @@ class ModelView(Model):
                 cls.__rpc__.setdefault(button,
                     RPC(instantiate=0, result=on_change_result))
 
-        # Update depend on methods
-        for (field_name, attribute), others in (
-                cls.__depend_methods.iteritems()):
-            field = getattr(cls, field_name)
-            for other in others:
-                other_field = getattr(cls, other)
-                setattr(field, attribute,
-                    getattr(field, attribute)
-                    | getattr(other_field, attribute))
-
     @classmethod
     def fields_view_get(cls, view_id=None, view_type='form'):
         '''
@@ -231,7 +242,7 @@ class ModelView(Model):
                     ],
                 ]
             views = View.search(domain)
-            views = filter(lambda v: v.rng_type == view_type, views)
+            views = [v for v in views if v.rng_type == view_type]
             if views:
                 view = views[0]
         if view:
@@ -379,12 +390,12 @@ class ModelView(Model):
 
         # Find field without read access
         fread_accesses = FieldAccess.check(cls.__name__,
-                cls._fields.keys(), 'read', access=True)
+                list(cls._fields.keys()), 'read', access=True)
         fields_to_remove = set(
             x for x, y in fread_accesses.items() if not y)
 
         # Find relation field without read access
-        for name, field in cls._fields.iteritems():
+        for name, field in cls._fields.items():
             if not ModelAccess.check_relation(cls.__name__, name, mode='read'):
                 fields_to_remove.add(name)
 
@@ -397,22 +408,33 @@ class ModelView(Model):
                         fields_to_remove.add(name)
             checked |= to_check
 
-        # Remove field without read access
-        for field in fields_to_remove:
-            xpath = ('//field[@name="%(field)s"] | //label[@name="%(field)s"]'
-                ' | //page[@name="%(field)s"] | //group[@name="%(field)s"]'
-                ' | //separator[@name="%(field)s"]') % {'field': field}
-            for i, element in enumerate(tree.xpath(xpath)):
-                if type == 'tree' or element.tag == 'page':
-                    parent = element.getparent()
-                    parent.remove(element)
-                elif type == 'form':
-                    element.tag = 'label'
-                    colspan = element.attrib.get('colspan')
-                    element.attrib.clear()
-                    element.attrib['id'] = 'hidden %s-%s' % (field, i)
-                    if colspan is not None:
-                        element.attrib['colspan'] = colspan
+        buttons_to_remove = set()
+        for name, definition in cls._buttons.items():
+            if fields_to_remove & set(definition.get('depends', [])):
+                buttons_to_remove.add(name)
+
+        field_xpath = ('//field[@name="%(name)s"]'
+            '| //label[@name="%(name)s"] | //page[@name="%(name)s"]'
+            '| //group[@name="%(name)s"] | //separator[@name="%(name)s"]')
+        button_xpath = '//button[@name="%(name)s"]'
+        # Remove field and button without read acces
+        for xpath, names in (
+                (field_xpath, fields_to_remove),
+                (button_xpath, buttons_to_remove),
+                ):
+            for name in names:
+                path = xpath % {'name': name}
+                for i, element in enumerate(tree.xpath(path)):
+                    if type == 'tree' or element.tag == 'page':
+                        parent = element.getparent()
+                        parent.remove(element)
+                    elif type == 'form':
+                        element.tag = 'label'
+                        colspan = element.attrib.get('colspan')
+                        element.attrib.clear()
+                        element.attrib['id'] = 'hidden %s-%s' % (name, i)
+                        if colspan is not None:
+                            element.attrib['colspan'] = colspan
 
         # Remove empty pages
         if type == 'form':
@@ -432,6 +454,9 @@ class ModelView(Model):
         fields_def = cls.__view_look_dom(tree_root, type,
                 fields_width=fields_width)
 
+        if hasattr(cls, 'active'):
+            fields_def.setdefault('active', {'name': 'active'})
+
         if field_children:
             fields_def.setdefault(field_children, {'name': field_children})
             if field_children in cls._fields:
@@ -439,7 +464,7 @@ class ModelView(Model):
                 if hasattr(field, 'field'):
                     fields_def.setdefault(field.field, {'name': field.field})
 
-        for field_name in fields_def.keys():
+        for field_name in list(fields_def.keys()):
             if field_name in cls._fields:
                 field = cls._fields[field_name]
             else:
@@ -447,9 +472,6 @@ class ModelView(Model):
             for depend in field.depends:
                 fields_def.setdefault(depend, {'name': depend})
 
-        if 'active' in cls._fields:
-            fields_def.setdefault('active', {'name': 'active'})
-
         arch = etree.tostring(
             tree, encoding='utf-8', pretty_print=False).decode('utf-8')
         # Do not call fields_def without fields as it returns all fields
@@ -547,6 +569,11 @@ class ModelView(Model):
             else:
                 states = {}
             groups = set(User.get_groups())
+            button_attr = Button.get_view_attributes(
+                cls.__name__, button_name)
+            for attr, value in button_attr.items():
+                if not element.get(attr):
+                    element.set(attr, value or '')
             button_groups = Button.get_groups(cls.__name__, button_name)
             if ((button_groups and not groups & button_groups)
                     or (not button_groups
@@ -568,6 +595,9 @@ class ModelView(Model):
             else:
                 element.set('type', 'instance')
 
+            for depend in states.get('depends', []):
+                fields_attrs.setdefault(depend, {})
+
         # translate view
         if Transaction().language != 'en':
             for attr in ('string', 'sum', 'confirm', 'help'):
@@ -625,10 +655,9 @@ class ModelView(Model):
                     if button_rules:
                         clicks = ButtonClick.register(
                             cls.__name__, func.__name__, records)
-                        records = filter(
-                            lambda r: all(br.test(r, clicks.get(r.id, []))
-                                for br in button_rules),
-                            records)
+                        records = [r for r in records
+                            if all(br.test(r, clicks.get(r.id, []))
+                                for br in button_rules)]
                 # Reset click after filtering in case the button also has rules
                 names = Button.get_reset(cls.__name__, func.__name__)
                 if names:
@@ -699,7 +728,7 @@ class ModelView(Model):
         init_values = self._init_values or {}
         if not self._values:
             return changed
-        for fname, value in self._values.iteritems():
+        for fname, value in self._values.items():
             field = self._fields[fname]
             # Always test key presence in case value is None
             if (fname in init_values
