@@ -9,8 +9,10 @@ __all__ = [
     'ModelViewChangedValues',
     'ModelViewChangedValuesTarget',
     'ModelViewButton',
+    'ModelViewButtonDepends',
     'ModelViewRPC',
     'ModelViewEmptyPage',
+    'ModelViewCircularDepends',
     ]
 
 
@@ -57,6 +59,26 @@ class ModelViewButton(ModelView):
         pass
 
 
+class ModelViewButtonDepends(ModelView):
+    'ModelView Button Depends'
+    __name__ = 'test.modelview.button_depends'
+    value = fields.Integer("Value")
+
+    @classmethod
+    def __setup__(cls):
+        super(ModelViewButtonDepends, cls).__setup__()
+        cls._buttons = {
+            'test': {
+                'depends': ['value'],
+                },
+            }
+
+    @classmethod
+    @ModelView.button
+    def test(cls, records):
+        pass
+
+
 class ModelViewRPC(ModelView):
     'ModelView RPC'
     __name__ = 'test.modelview.rpc'
@@ -112,3 +134,12 @@ class ModelViewRPC(ModelView):
 class ModelViewEmptyPage(ModelView):
     'ModelView Empty Page'
     __name__ = 'test.modelview.empty_page'
+
+
+class ModelViewCircularDepends(ModelView):
+    'ModelView Circular Depends'
+    __name__ = 'test.modelview.circular_depends'
+
+    foo = fields.Char("Char", depends=['bar'])
+    bar = fields.Char("Char", depends=['foobar'])
+    foobar = fields.Char("Char", depends=['foo'])
